<?php


namespace Buckhill\U2F\tests\units\Enroll;

require('vendor/autoload.php');

use mageekguy\atoum;

class Request extends atoum
{
    public function testHello()
    {
        $this->given($enrollment = new \Buckhill\U2F\Enroll\Request('https://buckhill.co.uk'))
            ->then
                ->object($enrollment)->isInstanceOf(\Buckhill\U2F\Enroll\Request::class)
                ->array($enrollment->toArray())->isNotEmpty()
            ;

        return 1;
    }
}